@@ -17,9 +17,6 @@
 
 import difflib
 import os
-import logging
-
-import libvirt
 
 import virtinst
 import virtinst.cli
@@ -40,19 +37,21 @@ uri_test_remote = uri_test + ",remote"
 
 _uri_qemu = "%s,qemu" % uri_test
 _uri_kvm_domcaps = (_uri_qemu + _domcapsprefix + "kvm-x86_64-domcaps.xml")
+_uri_kvm_aarch64_domcaps = (_uri_qemu + _domcapsprefix + "kvm-aarch64-domcaps.xml")
 uri_kvm_nodomcaps = (_uri_qemu + _capsprefix + "kvm-x86_64.xml")
 uri_kvm_rhel = (_uri_kvm_domcaps + _capsprefix + "kvm-x86_64-rhel7.xml")
 uri_kvm = (_uri_kvm_domcaps + _capsprefix + "kvm-x86_64.xml")
 uri_kvm_session = uri_kvm + ",session"
 
 uri_kvm_armv7l = (_uri_kvm_domcaps + _capsprefix + "kvm-armv7l.xml")
-uri_kvm_aarch64 = (_uri_kvm_domcaps + _capsprefix + "kvm-aarch64.xml")
+uri_kvm_aarch64 = (_uri_kvm_aarch64_domcaps + _capsprefix + "kvm-aarch64.xml")
 uri_kvm_ppc64le = (_uri_kvm_domcaps + _capsprefix + "kvm-ppc64le.xml")
 uri_kvm_s390x = (_uri_kvm_domcaps + _capsprefix + "kvm-s390x.xml")
 uri_kvm_s390x_KVMIBM = (_uri_kvm_domcaps + _capsprefix + "kvm-s390x-KVMIBM.xml")
 
 uri_xen = uri_test + _capsprefix + "xen-rhel5.4.xml,xen"
 uri_lxc = uri_test + _capsprefix + "lxc.xml,lxc"
+uri_vz = uri_test + _capsprefix + "vz.xml,vz"
 
 
 def get_debug():
@@ -141,11 +140,6 @@ def open_test_remote():
     return openconn(uri_test_remote)
 
 
-def _libvirt_callback(ignore, err):
-    logging.warn("libvirt errmsg: %s", err[2])
-libvirt.registerErrorHandler(f=_libvirt_callback, ctx=None)
-
-
 def test_create(testconn, xml, define_func="defineXML"):
     xml = virtinst.uri.sanitize_xml_for_test_define(xml)
 
