@@ -98,10 +98,10 @@ class vmmInterface(vmmLibvirtObject):
         elif itype:
             return str(itype).capitalize()
         else:
-            return "Interface"
+            return _("Interface")
 
     def get_startmode(self):
-        return self.get_xmlobj().start_mode or "none"
+        return self.get_xmlobj(inactive=True).start_mode or "none"
 
     def set_startmode(self, newmode):
         xmlobj = self._make_xmlobj_to_define()
@@ -109,7 +109,7 @@ class vmmInterface(vmmLibvirtObject):
         self._redefine_xmlobj(xmlobj)
 
     def get_slaves(self):
-        return [[obj.name, obj.type or "Unknown"] for obj in
+        return [[obj.name, obj.type or _("Unknown")] for obj in
                 self.get_xmlobj().interfaces]
 
     def get_slave_names(self):
@@ -117,24 +117,35 @@ class vmmInterface(vmmLibvirtObject):
         return [x[0] for x in self.get_slaves()]
 
     def _get_ip(self, iptype):
-        obj = self.get_xmlobj()
-        found = None
-        for protocol in obj.protocols:
+        # Get list of IP addresses from active XML and protocol configuration
+        # from inactive XML to figure out whether the IP address is static or
+        # from DHCP server.
+        activeObj = self.get_xmlobj()
+        inactiveObj = self.get_xmlobj(inactive=True)
+
+        activeProto = None
+        inactiveProto = None
+        for protocol in activeObj.protocols:
             if protocol.family == iptype:
-                found = protocol
+                activeProto = protocol
                 break
-        if not found:
+        for protocol in inactiveObj.protocols:
+            if protocol.family == iptype:
+                inactiveProto = protocol
+                break
+
+        if not activeProto and not inactiveProto:
             return None, []
 
         ret = []
-        for ip in found.ips:
+        for ip in activeProto.ips:
             ipstr = ip.address
             if not ipstr:
                 continue
             if ip.prefix:
                 ipstr += "/%s" % ip.prefix
             ret.append(ipstr)
-        return found, ret
+        return inactiveProto or activeProto, ret
 
     def get_ipv4(self):
         proto, ips = self._get_ip("ipv4")
