@@ -39,6 +39,7 @@ from .devicedisk import VirtualDisk
 from .devicegraphics import VirtualGraphics
 from .deviceinput import VirtualInputDevice
 from .deviceredirdev import VirtualRedirDevice
+from .devicerng import VirtualRNGDevice
 from .devicevideo import VirtualVideoDevice
 from .distroinstaller import DistroInstaller
 from .domainblkiotune import DomainBlkiotune
@@ -52,7 +53,9 @@ from .idmap import IdMap
 from .osxml import OSXML
 from .pm import PM
 from .seclabel import Seclabel
+from .sysinfo import SYSInfo
 from .xmlbuilder import XMLBuilder, XMLProperty, XMLChildProperty
+from .xmlnsqemu import XMLNSQemu
 
 
 class Guest(XMLBuilder):
@@ -104,9 +107,9 @@ class Guest(XMLBuilder):
     _XML_PROP_ORDER = ["type", "name", "uuid", "title", "description",
         "maxmemory", "memory", "blkiotune", "memtune", "memoryBacking",
         "vcpus", "curvcpus", "vcpu_placement", "cpuset",
-        "numatune", "bootloader", "os", "idmap",
+        "numatune", "resource", "sysinfo", "bootloader", "os", "idmap",
         "features", "cpu", "clock", "on_poweroff", "on_reboot", "on_crash",
-        "resource", "pm", "emulator", "_devices", "seclabel"]
+        "pm", "emulator", "_devices", "seclabels"]
 
     def __init__(self, *args, **kwargs):
         XMLBuilder.__init__(self, *args, **kwargs)
@@ -122,6 +125,7 @@ class Guest(XMLBuilder):
         self.skip_default_sound = False
         self.skip_default_usbredir = False
         self.skip_default_graphics = False
+        self.skip_default_rng = False
         self.x86_cpu_default = self.cpu.SPECIAL_MODE_HOST_MODEL_ONLY
 
         self.__os_object = None
@@ -187,6 +191,7 @@ class Guest(XMLBuilder):
                        validate_cb=lambda s, v: util.validate_uuid(v),
                        default_cb=_get_default_uuid)
 
+
     id = XMLProperty("./@id", is_int=True)
     type = XMLProperty("./@type", default_cb=lambda s: "xen")
     bootloader = XMLProperty("./bootloader")
@@ -194,13 +199,12 @@ class Guest(XMLBuilder):
     title = XMLProperty("./title")
     emulator = XMLProperty("./devices/emulator")
 
-    on_poweroff = XMLProperty("./on_poweroff",
-                              default_cb=lambda s: "destroy")
-    on_reboot = XMLProperty("./on_reboot", default_cb=lambda s: "restart")
-    on_crash = XMLProperty("./on_crash", default_cb=lambda s: "restart")
+    on_poweroff = XMLProperty("./on_poweroff")
+    on_reboot = XMLProperty("./on_reboot")
+    on_crash = XMLProperty("./on_crash")
     on_lockfailure = XMLProperty("./on_lockfailure")
 
-    seclabel = XMLChildProperty(Seclabel)
+    seclabels = XMLChildProperty(Seclabel)
     os = XMLChildProperty(OSXML, is_single=True)
     features = XMLChildProperty(DomainFeatures, is_single=True)
     clock = XMLChildProperty(Clock, is_single=True)
@@ -212,6 +216,9 @@ class Guest(XMLBuilder):
     memoryBacking = XMLChildProperty(DomainMemorybacking, is_single=True)
     idmap = XMLChildProperty(IdMap, is_single=True)
     resource = XMLChildProperty(DomainResource, is_single=True)
+    sysinfo = XMLChildProperty(SYSInfo, is_single=True)
+
+    xmlns_qemu = XMLChildProperty(XMLNSQemu, is_single=True)
 
 
     ###############################
@@ -311,7 +318,7 @@ class Guest(XMLBuilder):
         # We do a shallow copy of the OS block here, so that we can
         # set the install time properties but not permanently overwrite
         # any config the user explicitly requested.
-        data = (self.os, self.on_crash, self.on_reboot)
+        data = (self.os, self.on_reboot)
         try:
             self._propstore["os"] = self.os.copy()
         except:
@@ -321,7 +328,6 @@ class Guest(XMLBuilder):
 
     def _finish_get_xml(self, data):
         (self._propstore["os"],
-         self.on_crash,
          self.on_reboot) = data
 
     def _get_install_xml(self, *args, **kwargs):
@@ -331,33 +337,25 @@ class Guest(XMLBuilder):
         finally:
             self._finish_get_xml(data)
 
-    def _do_get_install_xml(self, install=True, disk_boot=False):
+    def _do_get_install_xml(self, install):
         """
         Return the full Guest xml configuration.
 
-        @param install: Whether we want the 'OS install' configuration or
-                        the 'post-install' configuration. (Some installs,
-                        like an import or livecd may not have an 'install'
-                        config.)
-        @type install: C{bool}
-        @param disk_boot: Whether we should boot off the harddisk, regardless
-                          of our position in the install process (this is
-                          used for 2 stage installs, where the second stage
-                          boots off the disk. You probably don't need to touch
-                          this.)
-        @type disk_boot: C{bool}
+        @install: Whether we want the 'OS install' configuration or
+            the 'post-install' configuration. The difference is mostly
+            whether the install media is attached and set as the boot
+            device. Some installs, like an import or livecd, do not have
+            an 'install' config.
         """
-        osblob_install = install and not disk_boot
-        if osblob_install and not self.installer.has_install_phase():
+        if install and not self.installer.has_install_phase():
             return None
 
-        self.installer.alter_bootconfig(self, osblob_install)
+        self.installer.alter_bootconfig(self, install)
         if not install:
             self._remove_cdrom_install_media()
 
         if install:
             self.on_reboot = "destroy"
-            self.on_crash = "destroy"
 
         self._set_osxml_defaults()
 
@@ -375,62 +373,53 @@ class Guest(XMLBuilder):
     # Private install helpers #
     ###########################
 
-    def _build_meter(self, meter, is_initial):
-        if is_initial:
-            meter_label = _("Creating domain...")
-        else:
-            meter_label = _("Starting domain...")
-
-        meter = util.ensure_meter(meter)
-        meter.start(size=None, text=meter_label)
-
-        return meter
-
-    def _build_xml(self, is_initial):
-        log_label = is_initial and "install" or "continue"
-        disk_boot = not is_initial
-
-        start_xml = self._get_install_xml(install=True, disk_boot=disk_boot)
+    def _build_xml(self):
+        install_xml = self._get_install_xml(install=True)
         final_xml = self._get_install_xml(install=False)
 
-        logging.debug("Generated %s XML: %s",
-                      log_label,
-                      (start_xml and ("\n" + start_xml) or "None required"))
+        logging.debug("Generated install XML: %s",
+            (install_xml and ("\n" + install_xml) or "None required"))
         logging.debug("Generated boot XML: \n%s", final_xml)
 
-        return start_xml, final_xml
+        return install_xml, final_xml
 
-    def _create_guest(self, meter,
-                      start_xml, final_xml, is_initial, noboot):
+    def _create_guest(self, meter, install_xml, final_xml, doboot, transient):
         """
         Actually do the XML logging, guest defining/creating
 
-        @param is_initial: If running initial guest creation, else we
-                           are continuing the install
-        @param noboot: Don't boot guest if no install phase
+        @param doboot: Boot guest even if it has no install phase
         """
-        meter = self._build_meter(meter, is_initial)
-        doboot = not noboot or self.installer.has_install_phase()
+        meter_label = _("Creating domain...")
+        meter = util.ensure_meter(meter)
+        meter.start(size=None, text=meter_label)
 
-        if is_initial and doboot:
-            dom = self.conn.createLinux(start_xml or final_xml, 0)
+        if transient:
+            self.domain = self.conn.createXML(install_xml or final_xml, 0)
         else:
-            dom = self.conn.defineXML(start_xml or final_xml)
-            if doboot:
-                dom.create()
+            # Not all hypervisors (vz) support createXML, so avoid it here
+            self.domain = self.conn.defineXML(install_xml or final_xml)
 
-        self.domain = dom
-        meter.end(0)
+            # Handle undefining the VM if the initial startup fails
+            if doboot or self.installer.has_install_phase():
+                try:
+                    self.domain.create()
+                except:
+                    import sys
+                    exc_info = sys.exc_info()
+                    try:
+                        self.domain.undefine()
+                    except:
+                        pass
+                    raise exc_info[0], exc_info[1], exc_info[2]
 
-        self.domain = self.conn.defineXML(final_xml)
-        if is_initial:
-            try:
-                logging.debug("XML fetched from libvirt object:\n%s",
-                              dom.XMLDesc(0))
-            except Exception, e:
-                logging.debug("Error fetching XML from libvirt object: %s", e)
+            if install_xml and install_xml != final_xml:
+                self.domain = self.conn.defineXML(final_xml)
 
-        return self.domain
+        try:
+            logging.debug("XML fetched from libvirt object:\n%s",
+                          self.domain.XMLDesc(0))
+        except Exception, e:
+            logging.debug("Error fetching XML from libvirt object: %s", e)
 
 
     def _flag_autostart(self):
@@ -455,20 +444,9 @@ class Guest(XMLBuilder):
     # Public API #
     ##############
 
-    def get_continue_inst(self):
-        """
-        Return True if this guest requires a call to 'continue_install',
-        which means the OS requires a 2 stage install (windows)
-        """
-        # If we are doing an 'import' or 'liveCD' install, there is
-        # no true install process, so continue install has no meaning
-        if not self.installer.has_install_phase():
-            return False
-
-        return self._os_object.is_windows()
-
     def start_install(self, meter=None,
-                      dry=False, return_xml=False, noboot=False):
+                      dry=False, return_xml=False,
+                      doboot=True, transient=False):
         """
         Begin the guest install (stage1).
         @param return_xml: Don't create the guest, just return generated XML
@@ -476,7 +454,6 @@ class Guest(XMLBuilder):
         if self.domain is not None:
             raise RuntimeError(_("Domain has already been started!"))
 
-        is_initial = True
         self.set_install_defaults()
 
         self._prepare_install(meter, dry)
@@ -486,9 +463,9 @@ class Guest(XMLBuilder):
                 for dev in self.get_all_devices():
                     dev.setup(meter)
 
-            start_xml, final_xml = self._build_xml(is_initial)
+            install_xml, final_xml = self._build_xml()
             if return_xml:
-                return (start_xml, final_xml)
+                return (install_xml, final_xml)
             if dry:
                 return
 
@@ -496,33 +473,14 @@ class Guest(XMLBuilder):
             self.check_vm_collision(self.conn, self.name,
                                     do_remove=self.replace)
 
-            self.domain = self._create_guest(meter,
-                                             start_xml, final_xml, is_initial,
-                                             noboot)
+            self._create_guest(meter, install_xml, final_xml,
+                               doboot, transient)
+
             # Set domain autostart flag if requested
             self._flag_autostart()
-
-            return self.domain
         finally:
             self.installer.cleanup()
 
-    def continue_install(self, meter=None,
-                         dry=False, return_xml=False):
-        """
-        Continue with stage 2 of a guest install. Only required for
-        guests which have the 'continue' flag set (accessed via
-        get_continue_inst)
-        """
-        is_initial = False
-        start_xml, final_xml = self._build_xml(is_initial)
-        if return_xml:
-            return (start_xml, final_xml)
-        if dry:
-            return
-
-        return self._create_guest(meter,
-                                  start_xml, final_xml, is_initial, False)
-
     def get_created_disks(self):
         return [d for d in self.get_devices("disk") if d.storage_was_created]
 
@@ -685,6 +643,25 @@ class Guest(XMLBuilder):
             return
         self.add_device(VirtualGraphics(self.conn))
 
+    def add_default_rng(self):
+        if self.skip_default_rng:
+            return
+        if self.get_devices("rng"):
+            return
+        if not self.os.is_x86():
+            # Not strictly x86 specific, but some other archs like
+            # arm have limited virtio options in some situations, so
+            # it needs more work there.
+            return
+
+        if (self.conn.is_qemu() and
+            self._os_object.supports_virtiorng() and
+            self.conn.check_support(self.conn.SUPPORT_CONN_RNG_URANDOM)):
+            dev = VirtualRNGDevice(self.conn)
+            dev.type = "random"
+            dev.device = "/dev/urandom"
+            self.add_device(dev)
+
     def add_default_devices(self):
         self.add_default_graphics()
         self.add_default_video_device()
@@ -692,6 +669,7 @@ class Guest(XMLBuilder):
         self.add_default_console_device()
         self.add_default_usb_controller()
         self.add_default_channels()
+        self.add_default_rng()
 
     def _add_install_cdrom(self):
         if self._install_cdrom_device:
@@ -708,11 +686,11 @@ class Guest(XMLBuilder):
     def _remove_cdrom_install_media(self):
         for dev in self.get_devices("disk"):
             # Keep the install cdrom device around, but with no media attached.
-            # But only if we are a distro that doesn't have a multi
-            # stage install (aka not Windows)
+            # But only if we are installing windows which has a multi stage
+            # install.
             if (dev.is_cdrom() and
                 getattr(dev, "installer_media", False) and
-                not self.get_continue_inst()):
+                not self._os_object.is_windows()):
                 dev.path = None
 
     def _set_defaults(self):
@@ -972,6 +950,8 @@ class Guest(XMLBuilder):
         for d in self.get_all_devices():
             if d.address.type != d.address.ADDRESS_TYPE_PCI:
                 continue
+            if None in [d.address.domain, d.address.bus, d.address.slot]:
+                continue
 
             addr = d.address
             addrstr = "%d%d%d" % (d.address.domain,
@@ -1036,7 +1016,10 @@ class Guest(XMLBuilder):
                     self._os_object.supports_virtiodisk()))):
                 d.bus = "virtio"
             elif self.os.is_pseries():
-                d.bus = "scsi"
+                if d.is_cdrom():
+                    d.bus = "scsi"
+                else:
+                    d.bus = 'virtio'
             elif self.os.is_arm():
                 d.bus = "sd"
             elif self.os.is_q35():
@@ -1084,15 +1067,14 @@ class Guest(XMLBuilder):
                 return False
             return all([c.model == "none" for c in controllers])
 
-        input_type = self._os_object.default_inputtype()
-        input_bus = self._os_object.default_inputbus()
+        input_type = "mouse"
+        input_bus = "ps2"
         if self.os.is_xenpv():
             input_type = VirtualInputDevice.TYPE_MOUSE
             input_bus = VirtualInputDevice.BUS_XEN
-        elif _usb_disabled() and input_bus == "usb":
-            input_bus = "ps2"
-            if input_type == "tablet":
-                input_type = "mouse"
+        elif self._os_object.supports_usbtablet() and not _usb_disabled():
+            input_type = "tablet"
+            input_bus = "usb"
 
         for inp in self.get_devices("input"):
             if (inp.type == inp.TYPE_DEFAULT and
@@ -1141,6 +1123,10 @@ class Guest(XMLBuilder):
                 if dev.image_compression is None:
                     dev.image_compression = "off"
 
+            if (dev.type == "spice" and dev.gl and
+                not self.conn.check_support(self.conn.SUPPORT_CONN_SPICE_GL)):
+                raise ValueError(_("Host does not support spice GL"))
+
     def _add_spice_channels(self):
         if self.skip_default_channel:
             return
@@ -1166,6 +1152,8 @@ class Guest(XMLBuilder):
             return
         if self.get_devices("redirdev"):
             return
+        if not self.os.is_x86():
+            return
         if not self.conn.check_support(self.conn.SUPPORT_CONN_USBREDIR):
             return
 
@@ -1183,6 +1171,11 @@ class Guest(XMLBuilder):
             if gfx.type == gfx.TYPE_SPICE:
                 return True
 
+    def has_gl(self):
+        for gfx in self.get_devices("graphics"):
+            if gfx.gl:
+                return True
+
     def _set_video_defaults(self):
         if self.has_spice():
             self._add_spice_channels()
@@ -1196,3 +1189,5 @@ class Guest(XMLBuilder):
         for video in self.get_devices("video"):
             if video.model == video.MODEL_DEFAULT:
                 video.model = video_model
+                if video.model == 'virtio' and self.has_gl():
+                    video.accel3d = True
